"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const XRObject_1 = __importDefault(require("../core/XRObject"));
const Cone_1 = __importDefault(require("../objects/Cone"));
const Cylinder_1 = __importDefault(require("../objects/Cylinder"));
class PlayerHand extends XRObject_1.default {
    constructor() {
        super(...arguments);
        this._triggerTouched = false;
    }
    init() {
        super.init();
        this._hand = new Cylinder_1.default({
            radiusTop: 0.01,
            radiusBottom: 0.01,
            height: 0.1,
            material: new three_1.MeshLambertMaterial({
                color: 0xffffff,
                transparent: true,
                opacity: 0.1,
            }),
        });
        this._hand.rotateX(three_1.MathUtils.degToRad(-90));
        this.add(this._hand);
        this._ray = new Cone_1.default({
            radius: 0.003,
            height: 0.4,
            material: new three_1.MeshLambertMaterial({
                color: 0xff0000,
                transparent: true,
                opacity: 0.1,
            }),
        });
        this._ray.position.set(0, this._ray.height / 2, 0);
        this.add(this._ray);
    }
    set pose(pose) {
        var _a, _b;
        const p = (_a = pose === null || pose === void 0 ? void 0 : pose.transform) === null || _a === void 0 ? void 0 : _a.position;
        const r = (_b = pose === null || pose === void 0 ? void 0 : pose.transform) === null || _b === void 0 ? void 0 : _b.orientation;
        if (p)
            this.position.set(p.x, p.y, p.z);
        if (r)
            this.quaternion.set(r.x, r.y, r.z, r.w);
    }
    set targetRayPose(pose) {
        var _a, _b;
        const p = (_a = pose === null || pose === void 0 ? void 0 : pose.transform) === null || _a === void 0 ? void 0 : _a.position;
        const r = (_b = pose === null || pose === void 0 ? void 0 : pose.transform) === null || _b === void 0 ? void 0 : _b.orientation;
        if (p) {
            this._ray.position.set(p.x, p.y, p.z - this._ray.height / 2);
            this._ray.position.sub(this.position);
        }
        if (r) {
            this._ray.quaternion.set(r.x, r.y, r.z, r.w);
            this._ray.quaternion.multiply(this.quaternion.clone().inverse());
            this._ray.rotateX(three_1.MathUtils.degToRad(-90));
        }
    }
    set triggerTouched(v) { }
}
exports.default = PlayerHand;
//# sourceMappingURL=PlayerHand.js.map