"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Spinning = void 0;
const three_1 = require("three");
const Walker_1 = __importDefault(require("../behaviours/walker/Walker"));
const XRObject_1 = __importDefault(require("../core/XRObject"));
const XRScene_1 = __importDefault(require("../core/XRScene"));
const Box_1 = __importDefault(require("../objects/Box"));
const PointLightBulb_1 = __importDefault(require("../objects/PointLightBulb"));
const BooleanParam_1 = __importDefault(require("../params/BooleanParam"));
const NumberParam_1 = __importDefault(require("../params/NumberParam"));
const randomVector3_1 = __importDefault(require("../utils/randomVector3"));
const rotateBy_1 = __importDefault(require("../utils/rotateBy"));
const setVector_1 = __importDefault(require("../utils/setVector"));
class Spinning extends XRScene_1.default {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { key: 'spinning', label: 'Spinning', controllerBehaviours: (0, Walker_1.default)(true), params: {
                bodyDistance: new NumberParam_1.default({
                    label: 'Body Distance',
                    defaultValue: 4,
                    min: 1,
                    max: 20,
                    step: 1,
                }),
                noOfLightBulbs: new NumberParam_1.default({
                    label: 'No of Lightbulbs',
                    defaultValue: 4,
                    min: 1,
                    max: 20,
                    step: 1,
                }),
                noOfBoxes: new NumberParam_1.default({
                    label: 'No of Boxes',
                    defaultValue: 30,
                    min: 1,
                    max: 500,
                    step: 10,
                }),
                boxSpeed: new NumberParam_1.default({
                    label: 'Box Speed',
                    defaultValue: 0.5,
                    min: 0.1,
                    max: 1,
                    step: 0.1,
                }),
                lightSpeed: new NumberParam_1.default({
                    label: 'Light Speed',
                    defaultValue: 0.5,
                    min: 0.1,
                    max: 1,
                    step: 0.1,
                }),
                boxColorRandom: new BooleanParam_1.default({ label: 'Random Box Color', defaultValue: true }),
                lightColorRandom: new BooleanParam_1.default({
                    label: 'Random Light Color',
                    defaultValue: true,
                }),
                boxSize: new NumberParam_1.default({
                    label: 'Box Size',
                    defaultValue: 1,
                    min: 0.1,
                    max: 10,
                    step: 0.1,
                }),
                lightDistance: new NumberParam_1.default({
                    label: 'Light Distance',
                    defaultValue: 5,
                    min: 1,
                    max: 10,
                    step: 1,
                }),
                lightDecay: new NumberParam_1.default({
                    label: 'Light Decay',
                    defaultValue: 1,
                    min: 1,
                    max: 4,
                    step: 1,
                }),
            } });
    }
    init() {
        super.init();
        for (let i = 0; i < this.params.noOfLightBulbs.value; i++) {
            const color = this.params.lightColorRandom.value ? Math.random() * 0xffffff : 0xffffff;
            const position = (0, randomVector3_1.default)(this.params.bodyDistance.value);
            const pointLight = new PointLightBulb_1.default({
                mass: 0,
                distance: this.params.lightDistance.value,
                decay: this.params.lightDecay.value,
                material: new three_1.MeshBasicMaterial({ color }),
            });
            const pointLightContainer = new XRObject_1.default({
                userData: {
                    speed: (0, randomVector3_1.default)(this.params.lightSpeed.value),
                },
            });
            pointLightContainer.add(pointLight);
            (0, setVector_1.default)(pointLight.position, position);
            this.addObject(pointLightContainer);
        }
        for (let i = 0; i < this.params.noOfBoxes.value; i++) {
            const position = (0, randomVector3_1.default)(this.params.bodyDistance.value);
            const size = (0, randomVector3_1.default)(this.params.boxSize.value);
            const box = new Box_1.default({
                mass: 0,
                width: size.x,
                height: size.y,
                depth: size.z,
                userData: {
                    speed: (0, randomVector3_1.default)(this.params.boxSpeed.value),
                },
                material: new three_1.MeshLambertMaterial({
                    color: this.params.boxColorRandom.value ? Math.random() * 0xffffff : 0xffffff,
                }),
            });
            (0, setVector_1.default)(box.position, position);
            this.addObject(box);
        }
    }
    update(p) {
        super.update(p);
        if (p === null || p === void 0 ? void 0 : p.delta) {
            this.objects.forEach(obj => {
                if (obj.userData.speed) {
                    (0, rotateBy_1.default)(obj, obj.userData.speed.clone().multiplyScalar(p.delta * Math.PI));
                }
            });
        }
    }
}
exports.Spinning = Spinning;
exports.default = Spinning;
//# sourceMappingURL=Spinning.js.map