"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ShapeFit_1 = __importDefault(require("../../enums/ShapeFit"));
const finishCollisionShape_1 = __importDefault(require("../finishCollisionShape"));
const setDefaultOptions_1 = __importDefault(require("./setDefaultOptions"));
function createHeightfieldTerrainShape(options = {}) {
    (0, setDefaultOptions_1.default)(options);
    if (options.fit === ShapeFit_1.default.ALL) {
        console.warn('cannot use fit: all with type: heightfield');
        return null;
    }
    const heightfieldDistance = options.heightfieldDistance || 1;
    const heightfieldData = options.heightfieldData || [];
    const heightScale = options.heightScale || 0;
    const upAxis = options.hasOwnProperty('upAxis') ? options.upAxis : 1;
    const hdt = (() => {
        switch (options.heightDataType) {
            case 'short':
                return Ammo['PHY_SHORT'];
            case 'float':
                return Ammo['PHY_FLOAT'];
            default:
                return Ammo['PHY_FLOAT'];
        }
    })();
    const flipQuadEdges = options.hasOwnProperty('flipQuadEdges') ? options.flipQuadEdges : true;
    const heightStickLength = heightfieldData.length;
    const heightStickWidth = heightStickLength > 0 ? heightfieldData[0].length : 0;
    const data = Ammo._malloc(heightStickLength * heightStickWidth * 4);
    const ptr = data / 4;
    let minHeight = Number.POSITIVE_INFINITY;
    let maxHeight = Number.NEGATIVE_INFINITY;
    let index = 0;
    for (let l = 0; l < heightStickLength; l++) {
        for (let w = 0; w < heightStickWidth; w++) {
            const height = heightfieldData[l][w];
            Ammo.HEAPF32[ptr + index] = height;
            index++;
            minHeight = Math.min(minHeight, height);
            maxHeight = Math.max(maxHeight, height);
        }
    }
    const collisionShape = new Ammo.btHeightfieldTerrainShape(heightStickWidth, heightStickLength, data, heightScale, minHeight, maxHeight, upAxis, hdt, flipQuadEdges);
    const scale = new Ammo.btVector3(heightfieldDistance, 1, heightfieldDistance);
    collisionShape.setLocalScaling(scale);
    Ammo.destroy(scale);
    collisionShape['heightfieldData'] = data;
    (0, finishCollisionShape_1.default)(collisionShape, options);
    return collisionShape;
}
exports.default = createHeightfieldTerrainShape;
//# sourceMappingURL=createHeightfieldTerrainShape.js.map