"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createHullShape = void 0;
const three_1 = require("three");
const ShapeFit_1 = __importDefault(require("../../enums/ShapeFit"));
const ShapeType_1 = __importDefault(require("../../enums/ShapeType"));
const computeBounds_1 = __importDefault(require("../computeBounds"));
const computeScale_1 = __importDefault(require("../computeScale"));
const finishCollisionShape_1 = __importDefault(require("../finishCollisionShape"));
const setDefaultOptions_1 = __importDefault(require("./setDefaultOptions"));
exports.createHullShape = (function () {
    const vertex = new three_1.Vector3();
    const center = new three_1.Vector3();
    const matrix = new three_1.Matrix4();
    return function (vertices, matrices, matrixWorld, options = {}) {
        options.type = ShapeType_1.default.HULL;
        (0, setDefaultOptions_1.default)(options);
        if (options.fit === ShapeFit_1.default.MANUAL) {
            console.warn('cannot use fit: manual with type: hull');
            return null;
        }
        const bounds = (0, computeBounds_1.default)(vertices, matrices);
        const btVertex = new Ammo.btVector3();
        const originalHull = new Ammo.btConvexHullShape();
        originalHull.setMargin(options.margin);
        center.addVectors(bounds.max, bounds.min).multiplyScalar(0.5);
        let vertexCount = 0;
        for (let i = 0; i < vertices.length; i++) {
            vertexCount += vertices[i].length / 3;
        }
        const maxVertices = options.hullMaxVertices || 100000;
        if (vertexCount > maxVertices) {
            console.warn(`too many vertices for hull shape; sampling ~${maxVertices} from ~${vertexCount} vertices`);
        }
        const p = Math.min(1, maxVertices / vertexCount);
        for (let i = 0; i < vertices.length; i++) {
            const components = vertices[i];
            matrix.fromArray(matrices[i]);
            for (let j = 0; j < components.length; j += 3) {
                const isLastVertex = i === vertices.length - 1 && j === components.length - 3;
                if (Math.random() <= p || isLastVertex) {
                    vertex
                        .set(components[j], components[j + 1], components[j + 2])
                        .applyMatrix4(matrix)
                        .sub(center);
                    btVertex.setValue(vertex.x, vertex.y, vertex.z);
                    originalHull.addPoint(btVertex, isLastVertex);
                }
            }
        }
        let collisionShape = originalHull;
        if (originalHull.getNumVertices() >= 100) {
            const shapeHull = new Ammo.btShapeHull(originalHull);
            shapeHull.buildHull(options.margin);
            Ammo.destroy(originalHull);
            collisionShape = new Ammo.btConvexHullShape(Ammo['getPointer'](shapeHull.getVertexPointer()), shapeHull.numVertices());
            Ammo.destroy(shapeHull);
        }
        Ammo.destroy(btVertex);
        (0, finishCollisionShape_1.default)(collisionShape, options, (0, computeScale_1.default)(matrixWorld, options));
        return collisionShape;
    };
})();
exports.default = exports.createHullShape;
//# sourceMappingURL=createHullShape.js.map